bug-gzip
[Top][All Lists]
Advanced

[Date Prev][Date Next][Thread Prev][Thread Next][Date Index][Thread Index]

Re: gzip: zfgrep fails to pass 2nd '-e' as an option, passes it as a fil


From: Paul Eggert
Subject: Re: gzip: zfgrep fails to pass 2nd '-e' as an option, passes it as a file
Date: Wed, 27 Dec 2006 00:08:05 -0800
User-agent: Gnus/5.1008 (Gnus v5.10.8) Emacs/21.4 (gnu/linux)

I believe I have fixed <http://bugs.debian.org/367673> upstream by
installing the following patch into gzip CVS:

2006-12-26  Paul Eggert  <address@hidden>

        Fix Debian bug 367673 <http://bugs.debian.org/367673> plus several
        other bugs discovered in the neighborhood.
        * NEWS: gzip -q now exits with status 2 (not 1) on SIGPIPE.
        zgrep has many bugs fixed with argument handling, special characters,
        and exit status.
        * gzip.c (abort_gzip_signal): Implement gzip -q change.
        * zgrep.in: Implement zgrep change.

Index: NEWS
===================================================================
RCS file: /cvsroot/gzip/gzip/NEWS,v
retrieving revision 1.12
diff -u -p -r1.12 NEWS
--- NEWS        24 Dec 2006 05:45:23 -0000      1.12
+++ NEWS        27 Dec 2006 08:00:40 -0000
@@ -1,9 +1,11 @@
-Gzip 1.3.10 (200?-??-??)
-
-* zcmp and zdiff did not work in the usual case, due to a typo.
+Major changes in Gzip 1.3.10 (200?-??-??)

 * gzip -c and zcat now work on special files, files with special mode bits,
   and files with multiple hard links.
+* gzip -q now exits with status 2 (not 1) when SIGPIPE is received.
+* zcmp and zdiff did not work in the usual case, due to a typo.
+* zgrep has many bugs fixed with argument handling, special characters,
+  and exit status.

 Gzip 1.3.9 (2006-12-15)

Index: zgrep.in
===================================================================
RCS file: /cvsroot/gzip/gzip/zgrep.in,v
retrieving revision 1.6
diff -u -p -r1.6 zgrep.in
--- zgrep.in    9 Dec 2006 04:25:56 -0000       1.6
+++ zgrep.in    27 Dec 2006 08:00:40 -0000
@@ -23,9 +23,9 @@
 PATH="BINDIR:$PATH"; export PATH

 case "$0" in
-       *egrep) grep=${EGREP-egrep}     ;;
-       *fgrep) grep=${FGREP-fgrep}     ;;
-       *)      grep=${GREP-grep}       ;;
+(*egrep)       grep='${EGREP-egrep}'   ;;
+(*fgrep)       grep='${FGREP-fgrep}'   ;;
+(*)            grep='${GREP-grep}'     ;;
 esac

 version="z$grep (gzip) @VERSION@
@@ -44,110 +44,144 @@ OPTIONs are the same as for '$grep'.

 Report bugs to <address@hidden>."

-pat=""
-after_dash_dash=""
+# sed script to escape all ' for the shell, and then (to handle trailing
+# newlines correctly) turn trailing X on last line into '.
+escape='
+  s/'\''/'\''\\'\'''\''/g
+  $s/X$/'\''/
+'
+have_pat=0
 files_with_matches=0
 files_without_matches=0
 no_filename=0
 with_filename=0

 while test $# -ne 0; do
-  case "$after_dash_dash$1" in
-  --d* | --rec*)       echo >&2 "$0: $1: option not supported"; exit 2;;
-  --h*)                        echo "$usage" || exit 2; exit;;
-  --files-with-*)      files_with_matches=1;;
-  --files-witho*)      files_without_matches=1;;
-  --no-f*)     no_filename=1;;
-  --v*)                echo "$version" || exit 2; exit;;
-  --wi*)       with_filename=1;;
-  --*) ;;
-  -*)
-       case "$1" in
-       -*[dr]*) echo >&2 "$0: $1: option not supported"; exit 2;;
-       esac
-       case "$1" in
-       -*H*)   with_filename=1;;
-       esac
-       case "$1" in
-       -*h*)   no_filename=1;;
-       esac
-       case "$1" in
-       -*L*)   files_without_matches=1;;
-       esac
-       case "$1" in
-       -*l*)   files_with_matches=1;;
-       esac;;
-  esac
-  case "$after_dash_dash$1" in
-  -[ef])   opt="$opt $1"; shift; pat="$1"
-           if test "$grep" = grep; then  # grep is buggy with -e on SVR4
-             grep=egrep
-           fi;;
-  -[ABCdm])opt="$opt $1 $2"; shift;;
-  --)      opt="$opt $1"; after_dash_dash=1;;
-  -*)     opt="$opt $1";;
-   *)      if test -z "$pat"; then
-            pat="$1"
-          else
-            break;
-           fi;;
+  option=$1
+  optarg=
+
+  case $1 in
+  (-[0123456789abcdhHiIKLlnoqrRsTuUvVwxyzZ]?*)
+    option=$(expr "X$1" : 'X\(-.[0-9]*\)')
+    arg2=-\'$(expr "X$1X" : 'X-.[0-9]*\(.*\)' | sed "$escape")
+    shift
+    eval "set X $arg2 "'${1+"$@"}';;
+  (--binary-*=* | --[lm]a*=* | --reg*=*)
+    ;;
+  (-[ABCDefm] | --binary-* | --file | --[lm]a* | --reg*)
+    have_optarg=1
+    case ${2?"$1 option requires an argument"} in
+    (*\'*)
+      optarg=" '"$(printf '%sX\n' "$2" | sed "$escape");;
+    (*)
+      optarg=" '$2'";;
+    esac
+    shift;;
+  (--)
+    shift
+    break;;
+  (-*)
+    ;;
+  (*)
+    break;;
   esac
   shift
+
+  case $option in
+  (-[drRzZ] | --di* | --exc* | --inc* | --rec* | --nu*)
+    printf >&2 '%s: %s: option not supported\n' "$0" "$option"
+    exit 2;;
+  (-[ef]* | --file | --file=* | --reg*)
+    have_pat=1;;
+  (--h | --he | --hel | --help)
+    echo "$usage" || exit 2
+    exit;;
+  (-H | --wi | --wit | --with | --with- | --with-f | --with-fi \
+  | --with-fil | --with-file | --with-filen | --with-filena | --with-filenam \
+  | --with-filename)
+    with_filename=1
+    continue;;
+  (-l | --files-with-*)
+    files_with_matches=1;;
+  (-L | --files-witho*)
+    files_without_matches=1;;
+  (--no-f*)
+    no_filename=1;;
+  (-V | --v | --ve | --ver | --vers | --versi | --versio | --version)
+    echo "$version" || exit 2
+    exit;;
+  esac
+
+  case $option in
+  (*\'?*)
+    option=\'$(expr "X${option}X" : 'X\(.*\)' | sed "$escape");;
+  (*)
+    option="'$option'";;
+  esac
+
+  grep="$grep $option$optarg"
 done

-if test -z "$pat"; then
-  echo "$usage"
-  exit 2
+if test $have_pat -eq 0; then
+  case ${1?"missing pattern; try \`$0 --help' for help"} in
+  (*\'*)
+    grep="$grep -- '"$(printf '%sX\n' "$1" | sed "$escape");;
+  (*)
+    grep="$grep -- '$1'";;
+  esac
+  shift
 fi

 if test $# -eq 0; then
-  gzip -cdfq | $grep $opt "$pat"
-  exit $?
+  set X -
+  shift
 fi

+exec 3>&1
 res=0
-for i do
-  gzip -cdfq -- "$i" |
+
+for i
+do
+  # Fail if gzip or grep (or sed) fails.
+  gzip_status=$(
+    exec 5>&1
+    (gzip -cdfq -- "$i" 5>&-; echo $? >&5) 3>&- |
     if test $files_with_matches -eq 1; then
-      $grep $opt "$pat" > /dev/null && printf '%s\n' "$i"
+      eval "$grep" >/dev/null && { printf '%s\n' "$i" || exit 2; }
     elif test $files_without_matches -eq 1; then
-      $grep $opt "$pat" > /dev/null || printf '%s\n' "$i"
-    elif test $with_filename -eq 0 && { test $# -eq 1 || test $no_filename -eq 
1; }; then
-      $grep $opt "$pat"
+      eval "$grep" >/dev/null || {
+       r=$?
+       if test $r -eq 1; then
+         printf '%s\n' "$i" || r=2
+       fi
+       exit $r
+      }
+    elif test $with_filename -eq 0 &&
+        { test $# -eq 1 || test $no_filename -eq 1; }; then
+      eval "$grep"
     else
-      escaped=
-      while :; do
-       case $i in
-       *'
-'*)
-         char='
-'        repl='\\n';;
-       *'&'*) char='&' repl='\&';;
-       *'\'*) char='\\' repl='\\';;
-       *'|'*) char='|' repl='\|';;
-       *) break;;
-       esac
-       up_to_first_char="\\([^$char]*\\)"
-       after_first_char="[^$char]*$char\\(.*\\)"
-       escaped=$escaped`expr "X$i" : "X$up_to_first_char"`$repl
-       i=`expr "X$i" : "$after_first_char"`
-      done
-      if test $with_filename -eq 1; then
-       sed_script="s|[^:]*|$escaped$i|"
-      else
-       sed_script="s|^|$escaped$i:|"
-      fi
-
-      # Fail if either grep or sed fails.
-      # Bash has ${PIPESTATUS[0]}, but that's not portable.
-      exec 3>&1
-      r=`
+      case $i in
+      (*'
+'* | *'&'* | *'\'* | *'|'*)
+        i=$(printf '%s\n' "$i" |
+           sed '
+             $!N
+             $s/[&\|]/\\&/g
+             $s/\n/\\n/g
+           ');;
+      esac
+      sed_script="s|^|$i:|"
+
+      # Fail if grep or sed fails.
+      r=$(
        exec 4>&1
-       ($grep $opt "$pat" 4>&-; echo $? >&4) 3>&- | sed "$sed_script" >&3 4>&-
-      ` &&
+       (eval "$grep" 4>&-; echo $? >&4) 3>&- | sed "$sed_script" >&3 4>&-
+      ) || r=2
       exit $r
-    fi
+    fi >&3 5>&-
+  )
   r=$?
+  test "$gzip_status" -eq 0 || test "$gzip_status" -eq 2 || r=2
   test $res -lt $r && res=$r
 done
 exit $res




reply via email to

[Prev in Thread] Current Thread [Next in Thread]
Index: gzip.c
===================================================================
RCS file: /cvsroot/gzip/gzip/gzip.c,v
retrieving revision 1.12
diff -u -p -r1.12 gzip.c
--- gzip.c      24 Dec 2006 05:45:23 -0000      1.12
+++ gzip.c      27 Dec 2006 08:00:40 -0000
@@ -218,7 +218,7 @@ size_t z_len;         /* strlen(z_suffix
 /* The set of signals that are caught.  */
 static sigset_t caught_signals;

 /* If nonzero then exit with status 1, rather than with the usual
    signal status, on receipt of a signal with this value.  This
    suppresses a "Broken Pipe" message with some shells.  */
 static int volatile exiting_signal;
@@ -1885,7 +1885,7 @@ abort_gzip_signal (sig)
     signal (sig, SIG_IGN);
    remove_output_file ();
    if (sig == exiting_signal)
-     _exit (ERROR);
+     _exit (WARNING);
    signal (sig, SIG_DFL);
    raise (sig);
 }
@@ -218,7 +218,7 @@ size_t z_len;         /* strlen(z_suffix
 static sigset_t caught_signals;

-/* If nonzero then exit with status 1, rather than with the usual
+/* If nonzero then exit with status WARNING, rather than with the usual
    signal status, on receipt of a signal with this value.  This
    suppresses a "Broken Pipe" message with some shells.  */